#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <misc.h>
#include <sstream.h>
#include "tlmplib.h"
#include "tlmplib.m"

void _F_loadfile::empty()
{
}
void _F_loadfile::start()
{
}
void _F_loadfile::end(off_t)
{
}
void _F_loadfile::missing(const char *fname)
{
	tlmp_error (MSG_U(E_CANTOPEN,"Can't open file %s (%s)\n")
		,fname,strerror(errno));
}

int loadfile (_F_loadfile &c, SSTREAM &ss, bool strip)
{
	c.priv = NULL;
	int ret = 0;
	char buf[1000];
	off_t lastpos = ss.getoffset();
	while (ss.gets(buf,sizeof(buf)-1)!=NULL){
		if (ret == 0) c.start();
		if (strip) strip_end (buf);
		if (c.oneline (buf,lastpos,ret)==-1){
			break;
		}
		lastpos = ss.getoffset();
		ret++;
	}
	if (ret == 0){
		c.empty();
	}else{
		c.end (ss.getoffset());
	}
	return ret;
}

int loadfile (_F_loadfile &c, const char *fname, bool strip)
{
	int ret = -1;
	if (strcmp(fname,"-")==0){
		SSTREAM_FILE ss(stdin);
		ret = loadfile (c,ss,strip);
	}else{
		FILE *fin = fopen(fname,"r");
		if (fin == NULL){
			c.missing (fname);
		}else{
			SSTREAM_FILE ss(fin);
			ret = loadfile (c,ss,strip);
			fclose (fin);
		}
	}
	return ret;
}
void _F_savefile::start(FILE *, const char *){}
void _F_savefile::end(FILE *, const char *){}
void _F_savefile::openfail(const char *fname)
{
	tlmp_error (MSG_R(E_CANTOPEN),fname,strerror(errno));
}


static int savefile_gen (_F_savefile &c, const char *fname, FILE *fout)
{
	c.priv = NULL;
	int ret = -1;
	if (fout == NULL){
		c.openfail (fname);
	}else{
		c.start(fout,fname);
		ret = c.dowrite (fout,fname);
		fflush (fout);
		c.end(fout,fname);
	}
	return ret;
}

int savefile (_F_savefile &c, const char *fname, bool append)
{
	FILE *fout = fopen(fname,append ? "a" : "w");
	int ret = savefile_gen (c,fname,fout);
	if (fout != NULL) if (fclose (fout) == -1) ret = -1;
	return ret;
}

int savefile (_F_savefile &c, CONFIG_FILE &cfg, bool append)
{
	FILE_CFG *fout = cfg.fopen(append ? "a" : "w");
	SSTRING tmp(cfg.getpath());
	int ret = savefile_gen (c,tmp.get(),filecfg_tofile(fout));
	if (fout != NULL) if (fclose (fout) == -1) ret = -1;
	return ret;
}

